@@ -264,8 +264,6 @@ class RespondToController < ActionController::Base
 end
 
 class RespondToControllerTest < ActionController::TestCase
-  tests RespondToController
-
   def setup
     super
     @request.host = "www.example.com"
@@ -581,10 +579,10 @@ class RespondToControllerTest < ActionController::TestCase
     end
 
     get :using_defaults
-    assert_equal "using_defaults - #{[:html].to_s}", @response.body
+    assert_equal "using_defaults - #{[:html]}", @response.body
 
     get :using_defaults, :format => "xml"
-    assert_equal "using_defaults - #{[:xml].to_s}", @response.body
+    assert_equal "using_defaults - #{[:xml]}", @response.body
   end
 
   def test_format_with_custom_response_type
