@@ -11,3 +11,10 @@ ActiveSupport::Deprecation.debug = true
 I18n.enforce_available_locales = false
 
 require 'active_support/testing/autorun'
+
+require 'mocha/setup' # FIXME: stop using mocha
+
+# FIXME: we have tests that depend on run order, we should fix that and
+# remove this method call.
+require 'active_support/test_case'
+ActiveSupport::TestCase.test_order = :sorted
