@@ -5,16 +5,20 @@ if ActiveRecord::Base.connection.supports_migrations?
   class ActiveRecordSchemaTest < ActiveRecord::TestCase
     self.use_transactional_fixtures = false
 
-    def setup
+    setup do
+      @original_verbose = ActiveRecord::Migration.verbose
+      ActiveRecord::Migration.verbose = false
       @connection = ActiveRecord::Base.connection
       ActiveRecord::SchemaMigration.drop_table
     end
 
-    def teardown
+    teardown do
       @connection.drop_table :fruits rescue nil
       @connection.drop_table :nep_fruits rescue nil
       @connection.drop_table :nep_schema_migrations rescue nil
+      @connection.drop_table :has_timestamps rescue nil
       ActiveRecord::SchemaMigration.delete_all rescue nil
+      ActiveRecord::Migration.verbose = @original_verbose
     end
 
     def test_has_no_primary_key
@@ -48,6 +52,7 @@ if ActiveRecord::Base.connection.supports_migrations?
 
     def test_schema_define_w_table_name_prefix
       table_name = ActiveRecord::SchemaMigration.table_name
+      old_table_name_prefix = ActiveRecord::Base.table_name_prefix
       ActiveRecord::Base.table_name_prefix  = "nep_"
       ActiveRecord::SchemaMigration.table_name = "nep_#{table_name}"
       ActiveRecord::Schema.define(:version => 7) do
@@ -60,7 +65,7 @@ if ActiveRecord::Base.connection.supports_migrations?
       end
       assert_equal 7, ActiveRecord::Migrator::current_version
     ensure
-      ActiveRecord::Base.table_name_prefix  = ""
+      ActiveRecord::Base.table_name_prefix  = old_table_name_prefix
       ActiveRecord::SchemaMigration.table_name = table_name
     end
 
@@ -87,5 +92,70 @@ if ActiveRecord::Base.connection.supports_migrations?
       assert_equal "017", ActiveRecord::SchemaMigration.normalize_migration_number("0017")
       assert_equal "20131219224947", ActiveRecord::SchemaMigration.normalize_migration_number("20131219224947")
     end
+
+    def test_timestamps_without_null_is_deprecated_on_create_table
+      assert_deprecated do
+        ActiveRecord::Schema.define do
+          create_table :has_timestamps do |t|
+            t.timestamps
+          end
+        end
+      end
+    end
+
+    def test_timestamps_without_null_is_deprecated_on_change_table
+      assert_deprecated do
+        ActiveRecord::Schema.define do
+          create_table :has_timestamps
+
+          change_table :has_timestamps do |t|
+            t.timestamps
+          end
+        end
+      end
+    end
+
+    def test_timestamps_without_null_is_deprecated_on_add_timestamps
+      assert_deprecated do
+        ActiveRecord::Schema.define do
+          create_table :has_timestamps
+          add_timestamps :has_timestamps
+        end
+      end
+    end
+
+    def test_no_deprecation_warning_from_timestamps_on_create_table
+      assert_not_deprecated do
+        ActiveRecord::Schema.define do
+          create_table :has_timestamps do |t|
+            t.timestamps null: true
+          end
+
+          drop_table :has_timestamps
+
+          create_table :has_timestamps do |t|
+            t.timestamps null: false
+          end
+        end
+      end
+    end
+
+    def test_no_deprecation_warning_from_timestamps_on_change_table
+      assert_not_deprecated do
+        ActiveRecord::Schema.define do
+          create_table :has_timestamps
+          change_table :has_timestamps do |t|
+            t.timestamps null: true
+          end
+
+          drop_table :has_timestamps
+
+          create_table :has_timestamps
+          change_table :has_timestamps do |t|
+            t.timestamps null: false, default: Time.now
+          end
+        end
+      end
+    end
   end
 end
