@@ -13,6 +13,9 @@ require 'models/mixed_case_monkey'
 require 'models/admin'
 require 'models/admin/account'
 require 'models/admin/user'
+require 'models/developer'
+require 'models/company'
+require 'models/project'
 
 class AutomaticInverseFindingTests < ActiveRecord::TestCase
   fixtures :ratings, :comments, :cars
@@ -198,6 +201,16 @@ class InverseAssociationTests < ActiveRecord::TestCase
     belongs_to_ref = Sponsor.reflect_on_association(:sponsor_club)
     assert_nil belongs_to_ref.inverse_of
   end
+
+  def test_this_inverse_stuff
+    firm = Firm.create!(name: 'Adequate Holdings')
+    Project.create!(name: 'Project 1', firm: firm)
+    Developer.create!(name: 'Gorbypuff', firm: firm)
+
+    new_project = Project.last
+    assert Project.reflect_on_association(:lead_developer).inverse_of.present?, "Expected inverse of to be present"
+    assert new_project.lead_developer.present?, "Expected lead developer to be present on the project"
+  end
 end
 
 class InverseHasOneTests < ActiveRecord::TestCase
@@ -356,7 +369,7 @@ class InverseHasManyTests < ActiveRecord::TestCase
 
   def test_parent_instance_should_be_shared_within_create_block_of_new_child
     man = Man.first
-    interest = man.interests.build do |i|
+    interest = man.interests.create do |i|
       assert i.man.equal?(man), "Man of child should be the same instance as a parent"
     end
     assert interest.man.equal?(man), "Man of the child should still be the same instance as a parent"
