@@ -1,6 +1,7 @@
 require "cases/helper"
 require 'models/computer'
 require 'models/developer'
+require 'models/computer'
 require 'models/project'
 require 'models/company'
 require 'models/categorization'
@@ -263,6 +264,11 @@ class AssociationProxyTest < ActiveRecord::TestCase
     end
   end
 
+  test "first! works on loaded associations" do
+    david = authors(:david)
+    assert_equal david.posts.first, david.posts.reload.first!
+  end
+
   def test_reset_unloads_target
     david = authors(:david)
     david.posts.reload
