@@ -1,4 +1,5 @@
 require "cases/helper"
+require 'models/aircraft'
 require 'models/post'
 require 'models/comment'
 require 'models/author'
@@ -7,6 +8,7 @@ require 'models/reply'
 require 'models/category'
 require 'models/company'
 require 'models/developer'
+require 'models/computer'
 require 'models/project'
 require 'models/minimalistic'
 require 'models/warehouse_thing'
@@ -19,7 +21,7 @@ require 'models/toy'
 require 'rexml/document'
 
 class PersistenceTest < ActiveRecord::TestCase
-  fixtures :topics, :companies, :developers, :projects, :computers, :accounts, :minimalistics, 'warehouse-things', :authors, :categorizations, :categories, :posts, :minivans, :pets, :toys
+  fixtures :topics, :companies, :developers, :projects, :computers, :accounts, :minimalistics, 'warehouse-things', :authors, :author_addresses, :categorizations, :categories, :posts, :minivans, :pets, :toys
 
   # Oracle UPDATE does not support ORDER BY
   unless current_adapter?(:OracleAdapter)
@@ -234,19 +236,12 @@ class PersistenceTest < ActiveRecord::TestCase
   end
 
   def test_save_with_duping_of_destroyed_object
-    developer = Developer.create(name: "Kuldeep")
+    developer = Developer.first
     developer.destroy
     new_developer = developer.dup
     new_developer.save
     assert new_developer.persisted?
-  end
-
-  def test_dup_of_destroyed_object_is_not_destroyed
-    developer = Developer.create(name: "Kuldeep")
-    developer.destroy
-    new_developer = developer.dup
-    new_developer.save
-    assert_equal new_developer.destroyed?, false
+    assert_not new_developer.destroyed?
   end
 
   def test_create_many
@@ -256,11 +251,9 @@ class PersistenceTest < ActiveRecord::TestCase
   end
 
   def test_create_columns_not_equal_attributes
-    topic = Topic.allocate.init_with(
-      'attributes' => {
-        'title'          => 'Another New Topic',
-        'does_not_exist' => 'test'
-      }
+    topic = Topic.instantiate(
+      'title'          => 'Another New Topic',
+      'does_not_exist' => 'test'
     )
     assert_nothing_raised { topic.save }
   end
@@ -306,10 +299,7 @@ class PersistenceTest < ActiveRecord::TestCase
     topic.title = "Still another topic"
     topic.save
 
-    topic_reloaded = Topic.allocate
-    topic_reloaded.init_with(
-      'attributes' => topic.attributes.merge('does_not_exist' => 'test')
-    )
+    topic_reloaded = Topic.instantiate(topic.attributes.merge('does_not_exist' => 'test'))
     topic_reloaded.title = 'A New Topic'
     assert_nothing_raised { topic_reloaded.save }
   end
@@ -477,7 +467,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_update_attribute_for_updated_at_on
     developer = Developer.find(1)
-    prev_month = Time.now.prev_month
+    prev_month = Time.now.prev_month.change(usec: 0)
 
     developer.update_attribute(:updated_at, prev_month)
     assert_equal prev_month, developer.updated_at
@@ -520,14 +510,14 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_update_column_should_not_leave_the_object_dirty
     topic = Topic.find(1)
-    topic.update_column("content", "Have a nice day")
+    topic.update_column("content", "--- Have a nice day\n...\n")
 
     topic.reload
-    topic.update_column(:content, "You too")
+    topic.update_column(:content, "--- You too\n...\n")
     assert_equal [], topic.changed
 
     topic.reload
-    topic.update_column("content", "Have a nice day")
+    topic.update_column("content", "--- Have a nice day\n...\n")
     assert_equal [], topic.changed
   end
 
@@ -548,7 +538,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_update_column_should_not_modify_updated_at
     developer = Developer.find(1)
-    prev_month = Time.now.prev_month
+    prev_month = Time.now.prev_month.change(usec: 0)
 
     developer.update_column(:updated_at, prev_month)
     assert_equal prev_month, developer.updated_at
@@ -611,14 +601,14 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_update_columns_should_not_leave_the_object_dirty
     topic = Topic.find(1)
-    topic.update({ "content" => "Have a nice day", :author_name => "Jose" })
+    topic.update({ "content" => "--- Have a nice day\n...\n", :author_name => "Jose" })
 
     topic.reload
-    topic.update_columns({ content: "You too", "author_name" => "Sebastian" })
+    topic.update_columns({ content: "--- You too\n...\n", "author_name" => "Sebastian" })
     assert_equal [], topic.changed
 
     topic.reload
-    topic.update_columns({ content: "Have a nice day", author_name: "Jose" })
+    topic.update_columns({ content: "--- Have a nice day\n...\n", author_name: "Jose" })
     assert_equal [], topic.changed
   end
 
@@ -645,7 +635,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_update_columns_should_not_modify_updated_at
     developer = Developer.find(1)
-    prev_month = Time.now.prev_month
+    prev_month = Time.now.prev_month.change(usec: 0)
 
     developer.update_columns(updated_at: prev_month)
     assert_equal prev_month, developer.updated_at
@@ -845,6 +835,20 @@ class PersistenceTest < ActiveRecord::TestCase
     end
   end
 
+  def test_persist_inherited_class_with_different_table_name
+    minimalistic_aircrafts = Class.new(Minimalistic) do
+      self.table_name = "aircraft"
+    end
+
+    assert_difference "Aircraft.count", 1 do
+      aircraft = minimalistic_aircrafts.create(name: "Wright Flyer")
+      aircraft.name = "Wright Glider"
+      aircraft.save
+    end
+
+    assert_equal "Wright Glider", Aircraft.last.name
+  end
+
   def test_instantiate_creates_a_new_instance
     post = Post.instantiate("title" => "appropriate documentation", "type" => "SpecialPost")
     assert_equal "appropriate documentation", post.title
@@ -856,6 +860,13 @@ class PersistenceTest < ActiveRecord::TestCase
     end
   end
 
+  def test_reload_removes_custom_selects
+    post = Post.select('posts.*, 1 as wibble').last!
+
+    assert_equal 1, post[:wibble]
+    assert_nil post.reload[:wibble]
+  end
+
   def test_find_via_reload
     post = Post.new
 
@@ -867,4 +878,31 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_equal "Welcome to the weblog", post.title
     assert_not post.new_record?
   end
+
+  def test_reload_via_querycache
+    ActiveRecord::Base.connection.enable_query_cache!
+    ActiveRecord::Base.connection.clear_query_cache
+    assert ActiveRecord::Base.connection.query_cache_enabled, 'cache should be on'
+    parrot = Parrot.create(:name => 'Shane')
+
+    # populate the cache with the SELECT result
+    found_parrot = Parrot.find(parrot.id)
+    assert_equal parrot.id, found_parrot.id
+
+    # Manually update the 'name' attribute in the DB directly
+    assert_equal 1, ActiveRecord::Base.connection.query_cache.length
+    ActiveRecord::Base.uncached do
+      found_parrot.name = 'Mary'
+      found_parrot.save
+    end
+
+    # Now reload, and verify that it gets the DB version, and not the querycache version
+    found_parrot.reload
+    assert_equal 'Mary', found_parrot.name
+
+    found_parrot = Parrot.find(parrot.id)
+    assert_equal 'Mary', found_parrot.name
+  ensure
+    ActiveRecord::Base.connection.disable_query_cache!
+  end
 end
