@@ -1,63 +1,64 @@
 class Author < ActiveRecord::Base
   has_many :posts
+  has_many :serialized_posts
+  has_one :post
   has_many :very_special_comments, :through => :posts
-  has_many :posts_with_comments, :include => :comments, :class_name => "Post"
-  has_many :popular_grouped_posts, :include => :comments, :class_name => "Post", :group => "type", :having => "SUM(comments_count) > 1", :select => "type"
-  has_many :posts_with_comments_sorted_by_comment_id, :include => :comments, :class_name => "Post", :order => 'comments.id'
-  has_many :posts_sorted_by_id_limited, :class_name => "Post", :order => 'posts.id', :limit => 1
-  has_many :posts_with_categories, :include => :categories, :class_name => "Post"
-  has_many :posts_with_comments_and_categories, :include => [ :comments, :categories ], :order => "posts.id", :class_name => "Post"
-  has_many :posts_containing_the_letter_a, :class_name => "Post"
-  has_many :posts_with_extension, :class_name => "Post" do #, :extend => ProxyTestExtension
-    def testing_proxy_owner
-      proxy_owner
-    end
-    def testing_proxy_reflection
-      proxy_reflection
-    end
-    def testing_proxy_target
-      proxy_target
+  has_many :posts_with_comments, -> { includes(:comments) }, :class_name => "Post"
+  has_many :popular_grouped_posts, -> { includes(:comments).group("type").having("SUM(comments_count) > 1").select("type") }, :class_name => "Post"
+  has_many :posts_with_comments_sorted_by_comment_id, -> { includes(:comments).order('comments.id') }, :class_name => "Post"
+  has_many :posts_sorted_by_id_limited, -> { order('posts.id').limit(1) }, :class_name => "Post"
+  has_many :posts_with_categories, -> { includes(:categories) }, :class_name => "Post"
+  has_many :posts_with_comments_and_categories, -> { includes(:comments, :categories).order("posts.id") }, :class_name => "Post"
+  has_many :posts_with_special_categorizations, :class_name => 'PostWithSpecialCategorization'
+  has_one  :post_about_thinking, -> { where("posts.title like '%thinking%'") }, :class_name => 'Post'
+  has_one  :post_about_thinking_with_last_comment, -> { where("posts.title like '%thinking%'").includes(:last_comment) }, :class_name => 'Post'
+  has_many :comments, through: :posts do
+    def ratings
+      Rating.joins(:comment).merge(self)
     end
   end
-  has_one  :post_about_thinking, :class_name => 'Post', :conditions => "posts.title like '%thinking%'"
-  has_one  :post_about_thinking_with_last_comment, :class_name => 'Post', :conditions => "posts.title like '%thinking%'", :include => :last_comment
-  has_many :comments, :through => :posts
   has_many :comments_containing_the_letter_e, :through => :posts, :source => :comments
-  has_many :comments_with_order_and_conditions, :through => :posts, :source => :comments, :order => 'comments.body', :conditions => "comments.body like 'Thank%'"
-  has_many :comments_with_include, :through => :posts, :source => :comments, :include => :post
+  has_many :comments_with_order_and_conditions, -> { order('comments.body').where("comments.body like 'Thank%'") }, :through => :posts, :source => :comments
+  has_many :comments_with_include, -> { includes(:post) }, :through => :posts, :source => :comments
 
   has_many :first_posts
-  has_many :comments_on_first_posts, :through => :first_posts, :source => :comments, :order => 'posts.id desc, comments.id asc'
+  has_many :comments_on_first_posts, -> { order('posts.id desc, comments.id asc') }, :through => :first_posts, :source => :comments
 
   has_one :first_post
-  has_one :comment_on_first_post,  :through => :first_post, :source => :comments, :order => 'posts.id desc, comments.id asc'
+  has_one :comment_on_first_post, -> { order('posts.id desc, comments.id asc') }, :through => :first_post, :source => :comments
 
-  has_many :thinking_posts, :class_name => 'Post', :conditions => { :title => 'So I was thinking' }, :dependent => :delete_all
-  has_many :welcome_posts,  :class_name => 'Post', :conditions => { :title => 'Welcome to the weblog' }
+  has_many :thinking_posts, -> { where(:title => 'So I was thinking') }, :dependent => :delete_all, :class_name => 'Post'
+  has_many :welcome_posts, -> { where(:title => 'Welcome to the weblog') }, :class_name => 'Post'
 
-  has_many :comments_desc, :through => :posts, :source => :comments, :order => 'comments.id DESC'
-  has_many :limited_comments, :through => :posts, :source => :comments, :limit => 1
+  has_many :welcome_posts_with_one_comment,
+           -> { where(title: 'Welcome to the weblog').where('comments_count = ?', 1) },
+           class_name: 'Post'
+  has_many :welcome_posts_with_comments,
+           -> { where(title: 'Welcome to the weblog').where(Post.arel_table[:comments_count].gt(0)) },
+           class_name: 'Post'
+
+  has_many :comments_desc, -> { order('comments.id DESC') }, :through => :posts, :source => :comments
   has_many :funky_comments, :through => :posts, :source => :comments
-  has_many :ordered_uniq_comments, :through => :posts, :source => :comments, :uniq => true, :order => 'comments.id'
-  has_many :ordered_uniq_comments_desc, :through => :posts, :source => :comments, :uniq => true, :order => 'comments.id DESC'
-  has_many :readonly_comments, :through => :posts, :source => :comments, :readonly => true
+  has_many :ordered_uniq_comments, -> { distinct.order('comments.id') }, :through => :posts, :source => :comments
+  has_many :ordered_uniq_comments_desc, -> { distinct.order('comments.id DESC') }, :through => :posts, :source => :comments
+  has_many :readonly_comments, -> { readonly }, :through => :posts, :source => :comments
 
   has_many :special_posts
   has_many :special_post_comments, :through => :special_posts, :source => :comments
+  has_many :special_posts_with_default_scope, :class_name => 'SpecialPostWithDefaultScope'
 
   has_many :sti_posts, :class_name => 'StiPost'
   has_many :sti_post_comments, :through => :sti_posts, :source => :comments
 
-  has_many :special_nonexistant_posts, :class_name => "SpecialPost", :conditions => "posts.body = 'nonexistant'"
-  has_many :special_nonexistant_post_comments, :through => :special_nonexistant_posts, :source => :comments, :conditions => "comments.post_id = 0"
+  has_many :special_nonexistant_posts, -> { where("posts.body = 'nonexistant'") }, :class_name => "SpecialPost"
+  has_many :special_nonexistant_post_comments, -> { where('comments.post_id' => 0) }, :through => :special_nonexistant_posts, :source => :comments
   has_many :nonexistant_comments, :through => :posts
 
-  has_many :hello_posts, :class_name => "Post", :conditions => "posts.body = 'hello'"
+  has_many :hello_posts, -> { where "posts.body = 'hello'" }, :class_name => "Post"
   has_many :hello_post_comments, :through => :hello_posts, :source => :comments
-  has_many :posts_with_no_comments, :class_name => 'Post', :conditions => 'comments.id is null', :include => :comments
+  has_many :posts_with_no_comments, -> { where('comments.id' => nil).includes(:comments) }, :class_name => 'Post'
 
-  has_many :hello_posts_with_hash_conditions, :class_name => "Post",
-:conditions => {:body => 'hello'}
+  has_many :hello_posts_with_hash_conditions, -> { where(:body => 'hello') }, :class_name => "Post"
   has_many :hello_post_comments_with_hash_conditions, :through =>
 :hello_posts_with_hash_conditions, :source => :comments
 
@@ -84,29 +85,31 @@ class Author < ActiveRecord::Base
   has_many :special_categories, :through => :special_categorizations, :source => :category
   has_one  :special_category,   :through => :special_categorizations, :source => :category
 
-  has_many :categories_like_general, :through => :categorizations, :source => :category, :class_name => 'Category', :conditions => { :name => 'General' }
+  has_many :categories_like_general, -> { where(:name => 'General') }, :through => :categorizations, :source => :category, :class_name => 'Category'
 
   has_many :categorized_posts, :through => :categorizations, :source => :post
-  has_many :unique_categorized_posts, :through => :categorizations, :source => :post, :uniq => true
+  has_many :unique_categorized_posts, -> { distinct }, :through => :categorizations, :source => :post
 
   has_many :nothings, :through => :kateggorisatons, :class_name => 'Category'
 
   has_many :author_favorites
-  has_many :favorite_authors, :through => :author_favorites, :order => 'name'
+  has_many :favorite_authors, -> { order('name') }, :through => :author_favorites
 
-  has_many :tagging,         :through => :posts
-  has_many :taggings,        :through => :posts
+  has_many :taggings,        :through => :posts, :source => :taggings
+  has_many :taggings_2,      :through => :posts, :source => :tagging
   has_many :tags,            :through => :posts
-  has_many :similar_posts,   :through => :tags,  :source => :tagged_posts, :uniq => true
-  has_many :distinct_tags,   :through => :posts, :source => :tags, :select => "DISTINCT tags.*", :order => "tags.name"
   has_many :post_categories, :through => :posts, :source => :categories
   has_many :tagging_tags,    :through => :taggings, :source => :tag
+
+  has_many :similar_posts, -> { distinct }, :through => :tags, :source => :tagged_posts
+  has_many :distinct_tags, -> { select("DISTINCT tags.*").order("tags.name") }, :through => :posts, :source => :tags
+
   has_many :tags_with_primary_key, :through => :posts
 
   has_many :books
   has_many :subscriptions,        :through => :books
-  has_many :subscribers,          :through => :subscriptions, :order => "subscribers.nick" # through has_many :through (on through reflection)
-  has_many :distinct_subscribers, :through => :subscriptions, :source => :subscriber, :select => "DISTINCT subscribers.*", :order => "subscribers.nick"
+  has_many :subscribers, -> { order("subscribers.nick") }, :through => :subscriptions
+  has_many :distinct_subscribers, -> { select("DISTINCT subscribers.*").order("subscribers.nick") }, :through => :subscriptions, :source => :subscriber
 
   has_one :essay, :primary_key => :name, :as => :writer
   has_one :essay_category, :through => :essay, :source => :category
@@ -130,18 +133,27 @@ class Author < ActiveRecord::Base
 
   has_many :category_post_comments, :through => :categories, :source => :post_comments
 
-  has_many :misc_posts, :class_name => 'Post',
-           :conditions => { :posts => { :title => ['misc post by bob', 'misc post by mary'] } }
+  has_many :misc_posts, -> { where(:posts => { :title => ['misc post by bob', 'misc post by mary'] }) }, :class_name => 'Post'
   has_many :misc_post_first_blue_tags, :through => :misc_posts, :source => :first_blue_tags
 
-  has_many :misc_post_first_blue_tags_2, :through => :posts, :source => :first_blue_tags_2,
-           :conditions => { :posts => { :title => ['misc post by bob', 'misc post by mary'] } }
+  has_many :misc_post_first_blue_tags_2, -> { where(:posts => { :title => ['misc post by bob', 'misc post by mary'] }) },
+           :through => :posts, :source => :first_blue_tags_2
 
   has_many :posts_with_default_include, :class_name => 'PostWithDefaultInclude'
   has_many :comments_on_posts_with_default_include, :through => :posts_with_default_include, :source => :comments
 
-  scope :relation_include_posts, includes(:posts)
-  scope :relation_include_tags, includes(:tags)
+  has_many :posts_with_signature, ->(record) { where("posts.title LIKE ?", "%by #{record.name.downcase}%") }, class_name: "Post"
+
+  has_many :posts_with_extension, -> { order(:title) }, class_name: "Post" do
+    def extension_method; end
+  end
+
+  has_many :posts_with_extension_and_instance, ->(record) { order(:title) }, class_name: "Post" do
+    def extension_method; end
+  end
+
+  scope :relation_include_posts, -> { includes(:posts) }
+  scope :relation_include_tags,  -> { includes(:tags) }
 
   attr_accessor :post_log
   after_initialize :set_post_log
