@@ -15,6 +15,8 @@ class Developer < ActiveRecord::Base
 
   accepts_nested_attributes_for :projects
 
+  has_and_belongs_to_many :shared_computers, class_name: "Computer"
+
   has_and_belongs_to_many :projects_extended_by_name,
       -> { extending(DeveloperProjectsAssociationExtension) },
       :class_name => "Project",
@@ -49,6 +51,9 @@ class Developer < ActiveRecord::Base
   has_many :comments, ->(developer) { where(body: "I'm #{developer.name}") }
   has_many :ratings, through: :comments
 
+  belongs_to :firm
+  has_many :contracted_projects, class_name: "Project"
+
   scope :jamises, -> { where(:name => 'Jamis') }
 
   validates_inclusion_of :salary, :in => 50000..200000
@@ -78,12 +83,6 @@ class AuditLog < ActiveRecord::Base
   belongs_to :unvalidated_developer, :class_name => 'Developer'
 end
 
-DeveloperSalary = Struct.new(:amount)
-class DeveloperWithAggregate < ActiveRecord::Base
-  self.table_name = 'developers'
-  composed_of :salary, :class_name => 'DeveloperSalary', :mapping => [%w(salary amount)]
-end
-
 class DeveloperWithBeforeDestroyRaise < ActiveRecord::Base
   self.table_name = 'developers'
   has_and_belongs_to_many :projects, :join_table => 'developers_projects', :foreign_key => 'developer_id'
