@@ -1,5 +1,4 @@
 class Liquid < ActiveRecord::Base
   self.table_name = :liquid
-  has_many :molecules, :uniq => true
+  has_many :molecules, -> { distinct }
 end
-
