@@ -1,7 +1,9 @@
 ActiveRecord::Schema.define do
 
-  %w(postgresql_tsvectors postgresql_hstores postgresql_arrays postgresql_moneys postgresql_numbers postgresql_times postgresql_network_addresses postgresql_bit_strings postgresql_uuids postgresql_ltrees
-      postgresql_oids postgresql_xml_data_type defaults geometrics postgresql_timestamp_with_zones postgresql_partitioned_table postgresql_partitioned_table_parent postgresql_json_data_type).each do |table_name|
+  %w(postgresql_tsvectors postgresql_hstores postgresql_arrays postgresql_moneys postgresql_numbers postgresql_times
+      postgresql_network_addresses postgresql_uuids postgresql_ltrees postgresql_oids postgresql_xml_data_type defaults
+      geometrics postgresql_timestamp_with_zones postgresql_partitioned_table postgresql_partitioned_table_parent
+      postgresql_citext).each do |table_name|
     execute "DROP TABLE IF EXISTS #{quote_table_name table_name}"
   end
 
@@ -62,8 +64,7 @@ _SQL
   CREATE TABLE postgresql_arrays (
     id SERIAL PRIMARY KEY,
     commission_by_quarter INTEGER[],
-    nicknames TEXT[],
-    name TEXT
+    nicknames TEXT[]
   );
 _SQL
 
@@ -100,22 +101,15 @@ _SQL
 _SQL
   end
 
-  if 't' == select_value("select 'json'=ANY(select typname from pg_type)")
+  if 't' == select_value("select 'citext'=ANY(select typname from pg_type)")
   execute <<_SQL
-  CREATE TABLE postgresql_json_data_type (
+  CREATE TABLE postgresql_citext (
     id SERIAL PRIMARY KEY,
-    json_data json default '{}'::json
+    text_citext citext default ''::citext
   );
 _SQL
   end
 
-  execute <<_SQL
-  CREATE TABLE postgresql_moneys (
-    id SERIAL PRIMARY KEY,
-    wealth MONEY
-  );
-_SQL
-
   execute <<_SQL
   CREATE TABLE postgresql_numbers (
     id SERIAL PRIMARY KEY,
@@ -141,14 +135,6 @@ _SQL
   );
 _SQL
 
-  execute <<_SQL
-  CREATE TABLE postgresql_bit_strings (
-    id SERIAL PRIMARY KEY,
-    bit_string BIT(8),
-    bit_string_varying BIT VARYING(8)
-  );
-_SQL
-
   execute <<_SQL
   CREATE TABLE postgresql_oids (
     id SERIAL PRIMARY KEY,
@@ -209,4 +195,8 @@ _SQL
     t.binary :binary, limit: 100_000
     t.text :text, limit: 100_000
   end
+
+  create_table :bigint_array, force: true do |t|
+    t.integer :big_int_data_points, limit: 8, array: true
+  end
 end
