@@ -9,13 +9,14 @@ ActiveRecord::Schema.define do
 
   #put adapter specific setup here
   case adapter_name
-    # For Firebird, set the sequence values 10000 when create_table is called;
-    # this prevents primary key collisions between "normally" created records
-    # and fixture-based (YAML) records.
-  when "Firebird"
-    def create_table(*args, &block)
-      ActiveRecord::Base.connection.create_table(*args, &block)
-      ActiveRecord::Base.connection.execute "SET GENERATOR #{args.first}_seq TO 10000"
+  when "PostgreSQL"
+    enable_extension!('uuid-ossp', ActiveRecord::Base.connection)
+    create_table :uuid_parents, id: :uuid, force: true do |t|
+      t.string :name
+    end
+    create_table :uuid_children, id: :uuid, force: true do |t|
+      t.string :name
+      t.uuid :uuid_parent_id
     end
   end
 
@@ -83,6 +84,8 @@ ActiveRecord::Schema.define do
   create_table :author_addresses, force: true do |t|
   end
 
+  add_foreign_key :authors, :author_addresses
+
   create_table :author_favorites, force: true do |t|
     t.column :author_id, :integer
     t.column :favorite_author_id, :integer
@@ -135,9 +138,11 @@ ActiveRecord::Schema.define do
     t.integer :engines_count
     t.integer :wheels_count
     t.column :lock_version, :integer, null: false, default: 0
-    t.timestamps
+    t.timestamps null: false
   end
 
+  create_table :carriers, force: true
+
   create_table :categories, force: true do |t|
     t.string :name, null: false
     t.string :type
@@ -189,7 +194,7 @@ ActiveRecord::Schema.define do
       t.text    :body, null: false
     end
     t.string  :type
-    t.integer :taggings_count, default: 0
+    t.integer :tags_count, default: 0
     t.integer :children_count, default: 0
     t.integer :parent_id
     t.references :author, polymorphic: true
@@ -225,6 +230,11 @@ ActiveRecord::Schema.define do
     t.integer :extendedWarranty, null: false
   end
 
+  create_table :computers_developers, id: false, force: true do |t|
+    t.references :computer
+    t.references :developer
+  end
+
   create_table :contracts, force: true do |t|
     t.integer :developer_id
     t.integer :company_id
@@ -239,6 +249,11 @@ ActiveRecord::Schema.define do
     t.string  :gps_location
   end
 
+  create_table :customer_carriers, force: true do |t|
+    t.references :customer
+    t.references :carrier
+  end
+
   create_table :dashboards, force: true, id: false do |t|
     t.string :dashboard_id
     t.string :name
@@ -247,6 +262,7 @@ ActiveRecord::Schema.define do
   create_table :developers, force: true do |t|
     t.string   :name
     t.integer  :salary, default: 70000
+    t.integer  :firm_id
     t.datetime :created_at
     t.datetime :updated_at
     t.datetime :created_on
@@ -273,6 +289,11 @@ ActiveRecord::Schema.define do
     t.string  :alias
   end
 
+  create_table :doubloons, force: true do |t|
+    t.integer :pirate_id
+    t.integer :weight
+  end
+
   create_table :edges, force: true, id: false do |t|
     t.column :source_id, :integer, null: false
     t.column :sink_id,   :integer, null: false
@@ -308,7 +329,7 @@ ActiveRecord::Schema.define do
   end
 
   create_table :cold_jokes, force: true do |t|
-    t.string :name
+    t.string :cold_name
   end
 
   create_table :friendships, force: true do |t|
@@ -535,7 +556,7 @@ ActiveRecord::Schema.define do
     t.references :best_friend_of
     t.integer    :insures, null: false, default: 0
     t.timestamp :born_at
-    t.timestamps
+    t.timestamps null: false
   end
 
   create_table :peoples_treasures, id: false, force: true do |t|
@@ -543,10 +564,16 @@ ActiveRecord::Schema.define do
     t.column :treasure_id, :integer
   end
 
+  create_table :personal_legacy_things, force: true do |t|
+    t.integer :tps_report_number
+    t.integer :person_id
+    t.integer :version, null: false, default: 0
+  end
+
   create_table :pets, primary_key: :pet_id, force: true do |t|
     t.string :name
     t.integer :owner_id, :integer
-    t.timestamps
+    t.timestamps null: false
   end
 
   create_table :pirates, force: true do |t|
@@ -569,7 +596,6 @@ ActiveRecord::Schema.define do
     end
     t.string  :type
     t.integer :comments_count, default: 0
-    t.integer :taggings_count, default: 0
     t.integer :taggings_with_delete_all_count, default: 0
     t.integer :taggings_with_destroy_count, default: 0
     t.integer :tags_count, default: 0
@@ -577,6 +603,16 @@ ActiveRecord::Schema.define do
     t.integer :tags_with_nullify_count, default: 0
   end
 
+  create_table :serialized_posts, force: true do |t|
+    t.integer :author_id
+    t.string :title, null: false
+  end
+
+  create_table :images, force: true do |t|
+    t.integer :imageable_identifier
+    t.string :imageable_class
+  end
+
   create_table :price_estimates, force: true do |t|
     t.string :estimate_of_type
     t.integer :estimate_of_id
@@ -596,6 +632,7 @@ ActiveRecord::Schema.define do
   create_table :projects, force: true do |t|
     t.string :name
     t.string :type
+    t.integer :firm_id
   end
 
   create_table :randomly_named_table, force: true do |t|
@@ -633,6 +670,8 @@ ActiveRecord::Schema.define do
     t.string :name
     t.integer :pirate_id
     t.integer :update_only_pirate_id
+    # Conventionally named column for counter_cache
+    t.integer :treasures_count, default: 0
     t.datetime :created_at
     t.datetime :created_on
     t.datetime :updated_at
@@ -644,6 +683,11 @@ ActiveRecord::Schema.define do
     t.integer :ship_id
   end
 
+  create_table :shop_accounts, force: true do |t|
+    t.references :customer
+    t.references :customer_carrier
+  end
+
   create_table :speedometers, force: true, id: false do |t|
     t.string :speedometer_id
     t.string :name
@@ -699,10 +743,14 @@ ActiveRecord::Schema.define do
   end
 
   create_table :topics, force: true do |t|
-    t.string   :title
+    t.string   :title, limit: 250
     t.string   :author_name
     t.string   :author_email_address
-    t.datetime :written_on
+    if mysql_56?
+      t.datetime :written_on, precision: 6
+    else
+      t.datetime :written_on
+    end
     t.time     :bonus_time
     t.date     :last_read
     # use VARCHAR2(4000) instead of CLOB datatype as CLOB data type has many limitations in
@@ -721,13 +769,13 @@ ActiveRecord::Schema.define do
     t.string   :parent_title
     t.string   :type
     t.string   :group
-    t.timestamps
+    t.timestamps null: true
   end
 
   create_table :toys, primary_key: :toy_id, force: true do |t|
     t.string :name
     t.integer :pet_id, :integer
-    t.timestamps
+    t.timestamps null: false
   end
 
   create_table :traffic_lights, force: true do |t|
@@ -743,6 +791,7 @@ ActiveRecord::Schema.define do
     t.column :type, :string
     t.column :looter_id, :integer
     t.column :looter_type, :string
+    t.belongs_to :ship
   end
 
   create_table :tyres, force: true do |t|
@@ -828,6 +877,11 @@ ActiveRecord::Schema.define do
     t.string 'from'
   end
 
+  create_table :trees, force: true do |t|
+    t.string :name
+    t.datetime :updated_at
+  end
+
   create_table :hotels, force: true do |t|
   end
   create_table :departments, force: true do |t|
@@ -841,6 +895,8 @@ ActiveRecord::Schema.define do
     t.integer :employable_id
     t.string :employable_type
     t.integer :department_id
+    t.string :employable_list_type
+    t.integer :employable_list_id
   end
 
   create_table :records, force: true do |t|
@@ -852,12 +908,22 @@ ActiveRecord::Schema.define do
       t.integer :fk_id, null: false
     end
 
-    create_table :fk_test_has_pk, force: true do |t|
+    create_table :fk_test_has_pk, force: true, primary_key: "pk_id" do |t|
     end
 
-    execute "ALTER TABLE fk_test_has_fk ADD CONSTRAINT fk_name FOREIGN KEY (#{quote_column_name 'fk_id'}) REFERENCES #{quote_table_name 'fk_test_has_pk'} (#{quote_column_name 'id'})"
+    add_foreign_key :fk_test_has_fk, :fk_test_has_pk, column: "fk_id", name: "fk_name", primary_key: "pk_id"
+    add_foreign_key :lessons_students, :students
+  end
 
-    execute "ALTER TABLE lessons_students ADD CONSTRAINT student_id_fk FOREIGN KEY (#{quote_column_name 'student_id'}) REFERENCES #{quote_table_name 'students'} (#{quote_column_name 'id'})"
+  create_table :overloaded_types, force: true do |t|
+    t.float :overloaded_float, default: 500
+    t.float :unoverloaded_float
+    t.string :overloaded_string_with_limit, limit: 255
+    t.string :string_with_default, default: 'the original default'
+  end
+
+  create_table :test_with_keyword_column_name, force: true do |t|
+    t.string :desc
   end
 end
 
@@ -869,3 +935,12 @@ end
 College.connection.create_table :colleges, force: true do |t|
   t.column :name, :string, null: false
 end
+
+Professor.connection.create_table :professors, force: true do |t|
+  t.column :name, :string, null: false
+end
+
+Professor.connection.create_table :courses_professors, id: false, force: true do |t|
+  t.references :course
+  t.references :professor
+end
