@@ -2,7 +2,7 @@ require 'abstract_unit'
 require 'active_support/core_ext/class'
 require 'set'
 
-class ClassTest < Test::Unit::TestCase
+class ClassTest < ActiveSupport::TestCase
   class Parent; end
   class Foo < Parent; end
   class Bar < Foo; end
