@@ -7,8 +7,7 @@ class SummablePayment < Payment
   def +(p) self.class.new(price + p.price) end
 end
 
-class EnumerableTests < Test::Unit::TestCase
-  Enumerator = [].each.class
+class EnumerableTests < ActiveSupport::TestCase
 
   class GenericEnumerable
     include Enumerable
@@ -21,28 +20,6 @@ class EnumerableTests < Test::Unit::TestCase
     end
   end
 
-  def test_group_by
-    names = %w(marcel sam david jeremy)
-    klass = Struct.new(:name)
-    objects = (1..50).inject([]) do |people,|
-      p = klass.new
-      p.name = names.sort_by { rand }.first
-      people << p
-    end
-
-    enum = GenericEnumerable.new(objects)
-    grouped = enum.group_by { |object| object.name }
-
-    grouped.each do |name, group|
-      assert group.all? { |person| person.name == name }
-    end
-
-    assert_equal objects.uniq.map(&:name), grouped.keys
-    assert({}.merge(grouped), "Could not convert ActiveSupport::OrderedHash into Hash")
-    assert_equal Enumerator, enum.group_by.class
-    assert_equal grouped, enum.group_by.each(&:name)
-  end
-
   def test_sums
     enum = GenericEnumerable.new([5, 15, 10])
     assert_equal 30, enum.sum
@@ -84,15 +61,11 @@ class EnumerableTests < Test::Unit::TestCase
     assert_equal 10, (1..4.5).sum
     assert_equal 6, (1...4).sum
     assert_equal 'abc', ('a'..'c').sum
-  end
-
-  def test_each_with_object
-    enum = GenericEnumerable.new(%w(foo bar))
-    result = enum.each_with_object({}) { |str, hsh| hsh[str] = str.upcase }
-    assert_equal({'foo' => 'FOO', 'bar' => 'BAR'}, result)
-    assert_equal Enumerator, enum.each_with_object({}).class
-    result2 = enum.each_with_object({}).each{|str, hsh| hsh[str] = str.upcase}
-    assert_equal result, result2
+    assert_equal 50_000_005_000_000, (0..10_000_000).sum
+    assert_equal 0, (10..0).sum
+    assert_equal 5, (10..0).sum(5)
+    assert_equal 10, (10..10).sum
+    assert_equal 42, (10...10).sum(42)
   end
 
   def test_index_by
@@ -100,6 +73,10 @@ class EnumerableTests < Test::Unit::TestCase
     assert_equal({ 5 => Payment.new(5), 15 => Payment.new(15), 10 => Payment.new(10) },
                  payments.index_by { |p| p.price })
     assert_equal Enumerator, payments.index_by.class
+    if Enumerator.method_defined? :size
+      assert_equal nil, payments.index_by.size
+      assert_equal 42, (1..42).index_by.size
+    end
     assert_equal({ 5 => Payment.new(5), 15 => Payment.new(15), 10 => Payment.new(10) },
                  payments.index_by.each { |p| p.price })
   end
