@@ -12,7 +12,7 @@ class RangeTest < ActiveSupport::TestCase
     date_range = Time.utc(2005, 12, 10, 15, 30)..Time.utc(2005, 12, 10, 17, 30)
     assert_equal "BETWEEN '2005-12-10 15:30:00' AND '2005-12-10 17:30:00'", date_range.to_s(:db)
   end
-  
+
   def test_date_range
     assert_instance_of Range, DateTime.new..DateTime.new
     assert_instance_of Range, DateTime::Infinity.new..DateTime::Infinity.new
@@ -115,6 +115,11 @@ class RangeTest < ActiveSupport::TestCase
 
   def test_date_time_with_each
     datetime = DateTime.now
-    assert ((datetime - 1.hour)..datetime).each {}
+    assert(((datetime - 1.hour)..datetime).each {})
+  end
+
+  def test_date_time_with_step
+    datetime = DateTime.now
+    assert(((datetime - 1.hour)..datetime).step(1) {})
   end
 end
