@@ -1,7 +1,9 @@
 require 'abstract_unit'
 require 'active_support/time'
+require 'time_zone_test_helpers'
 
 class TimeWithZoneTest < ActiveSupport::TestCase
+  include TimeZoneTestHelpers
 
   def setup
     @utc = Time.utc(2000, 1, 1, 0)
@@ -77,6 +79,11 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     assert_equal '1999-12-31 19:00:00 EST -0500', @twz.strftime('%Y-%m-%d %H:%M:%S %Z %z')
   end
 
+  def test_strftime_with_escaping
+    assert_equal '%Z %z', @twz.strftime('%%Z %%z')
+    assert_equal '%EST %-0500', @twz.strftime('%%%Z %%%z')
+  end
+
   def test_inspect
     assert_equal 'Fri, 31 Dec 1999 19:00:00 EST -05:00', @twz.inspect
   end
@@ -197,10 +204,14 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_eql?
+    assert_equal true, @twz.eql?(@twz.dup)
     assert_equal true, @twz.eql?(Time.utc(2000))
     assert_equal true, @twz.eql?( ActiveSupport::TimeWithZone.new(Time.utc(2000), ActiveSupport::TimeZone["Hawaii"]) )
     assert_equal false, @twz.eql?( Time.utc(2000, 1, 1, 0, 0, 1) )
     assert_equal false, @twz.eql?( DateTime.civil(1999, 12, 31, 23, 59, 59) )
+
+    other_twz = ActiveSupport::TimeWithZone.new(DateTime.now.utc, @time_zone)
+    assert_equal true, other_twz.eql?(other_twz.dup)
   end
 
   def test_hash
@@ -246,16 +257,31 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     assert_equal  86_400.0,  ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 2), ActiveSupport::TimeZone['Hawaii'] ) - Time.utc(2000, 1, 1)
   end
 
+  def test_minus_with_time_precision
+    assert_equal  86_399.999999998,  ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 2, 23, 59, 59, Rational(999999999, 1000)), ActiveSupport::TimeZone['UTC'] ) - Time.utc(2000, 1, 2, 0, 0, 0, Rational(1, 1000))
+    assert_equal  86_399.999999998,  ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 2, 23, 59, 59, Rational(999999999, 1000)), ActiveSupport::TimeZone['Hawaii'] ) - Time.utc(2000, 1, 2, 0, 0, 0, Rational(1, 1000))
+  end
+
   def test_minus_with_time_with_zone
     twz1 = ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 1), ActiveSupport::TimeZone['UTC'] )
     twz2 = ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 2), ActiveSupport::TimeZone['UTC'] )
     assert_equal  86_400.0,  twz2 - twz1
   end
 
+  def test_minus_with_time_with_zone_precision
+    twz1 = ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 1, 0, 0, 0, Rational(1, 1000)), ActiveSupport::TimeZone['UTC'] )
+    twz2 = ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 1, 23, 59, 59, Rational(999999999, 1000)), ActiveSupport::TimeZone['UTC'] )
+    assert_equal  86_399.999999998,  twz2 - twz1
+  end
+
   def test_minus_with_datetime
     assert_equal  86_400.0,  ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 2), ActiveSupport::TimeZone['UTC'] ) - DateTime.civil(2000, 1, 1)
   end
 
+  def test_minus_with_datetime_precision
+    assert_equal  86_399.999999999,  ActiveSupport::TimeWithZone.new( Time.utc(2000, 1, 1, 23, 59, 59, Rational(999999999, 1000)), ActiveSupport::TimeZone['UTC'] ) - DateTime.civil(2000, 1, 1)
+  end
+
   def test_minus_with_wrapped_datetime
     assert_equal  86_400.0,  ActiveSupport::TimeWithZone.new( DateTime.civil(2000, 1, 2), ActiveSupport::TimeZone['UTC'] ) - Time.utc(2000, 1, 1)
     assert_equal  86_400.0,  ActiveSupport::TimeWithZone.new( DateTime.civil(2000, 1, 2), ActiveSupport::TimeZone['UTC'] ) - DateTime.civil(2000, 1, 1)
@@ -350,6 +376,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_acts_like_time
+    assert @twz.acts_like_time?
     assert @twz.acts_like?(:time)
     assert ActiveSupport::TimeWithZone.new(DateTime.civil(2000), @time_zone).acts_like?(:time)
   end
@@ -795,23 +822,17 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     assert_equal "undefined method `this_method_does_not_exist' for Fri, 31 Dec 1999 19:00:00 EST -05:00:Time", e.message
     assert_no_match "rescue", e.backtrace.first
   end
-
-  protected
-    def with_env_tz(new_tz = 'US/Eastern')
-      old_tz, ENV['TZ'] = ENV['TZ'], new_tz
-      yield
-    ensure
-      old_tz ? ENV['TZ'] = old_tz : ENV.delete('TZ')
-    end
 end
 
 class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
+  include TimeZoneTestHelpers
+
   def setup
-    @t, @dt = Time.utc(2000), DateTime.civil(2000)
+    @t, @dt, @zone = Time.utc(2000), DateTime.civil(2000), Time.zone
   end
 
   def teardown
-    Time.zone = nil
+    Time.zone = @zone
   end
 
   def test_in_time_zone
@@ -867,8 +888,6 @@ class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
   def test_localtime
     Time.zone = ActiveSupport::TimeZone['Eastern Time (US & Canada)']
     assert_equal @dt.in_time_zone.localtime, @dt.in_time_zone.utc.to_time.getlocal
-  ensure
-    Time.zone = nil
   end
 
   def test_use_zone
@@ -907,6 +926,7 @@ class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
   end
 
   def test_time_zone_getter_and_setter_with_zone_default_set
+    old_zone_default = Time.zone_default
     Time.zone_default = ActiveSupport::TimeZone['Alaska']
     assert_equal ActiveSupport::TimeZone['Alaska'], Time.zone
     Time.zone = ActiveSupport::TimeZone['Hawaii']
@@ -914,8 +934,7 @@ class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
     Time.zone = nil
     assert_equal ActiveSupport::TimeZone['Alaska'], Time.zone
   ensure
-    Time.zone = nil
-    Time.zone_default = nil
+    Time.zone_default = old_zone_default
   end
 
   def test_time_zone_setter_is_thread_safe
@@ -987,8 +1006,6 @@ class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
       assert_equal 'Eastern Time (US & Canada)', Time.current.time_zone.name
       assert_equal Time.utc(2000), Time.current.time
     end
-  ensure
-    Time.zone = nil
   end
 
   def test_time_in_time_zone_doesnt_affect_receiver
@@ -999,25 +1016,15 @@ class TimeWithZoneMethodsForTimeAndDateTimeTest < ActiveSupport::TestCase
       assert_not time.utc?, 'time expected to be local, but is UTC'
     end
   end
-
-  protected
-    def with_env_tz(new_tz = 'US/Eastern')
-      old_tz, ENV['TZ'] = ENV['TZ'], new_tz
-      yield
-    ensure
-      old_tz ? ENV['TZ'] = old_tz : ENV.delete('TZ')
-    end
 end
 
 class TimeWithZoneMethodsForDate < ActiveSupport::TestCase
+  include TimeZoneTestHelpers
+
   def setup
     @d = Date.civil(2000)
   end
 
-  def teardown
-    Time.zone = nil
-  end
-
   def test_in_time_zone
     with_tz_default 'Alaska' do
       assert_equal 'Sat, 01 Jan 2000 00:00:00 AKST -09:00', @d.in_time_zone.inspect
@@ -1050,28 +1057,17 @@ class TimeWithZoneMethodsForDate < ActiveSupport::TestCase
     assert_raise(ArgumentError) { @d.in_time_zone(-15.hours) }
     assert_raise(ArgumentError) { @d.in_time_zone(Object.new) }
   end
-
-  protected
-    def with_tz_default(tz = nil)
-      old_tz = Time.zone
-      Time.zone = tz
-      yield
-    ensure
-      Time.zone = old_tz
-    end
 end
 
 class TimeWithZoneMethodsForString < ActiveSupport::TestCase
+  include TimeZoneTestHelpers
+
   def setup
     @s = "Sat, 01 Jan 2000 00:00:00"
     @u = "Sat, 01 Jan 2000 00:00:00 UTC +00:00"
     @z = "Fri, 31 Dec 1999 19:00:00 EST -05:00"
   end
 
-  def teardown
-    Time.zone = nil
-  end
-
   def test_in_time_zone
     with_tz_default 'Alaska' do
       assert_equal 'Sat, 01 Jan 2000 00:00:00 AKST -09:00', @s.in_time_zone.inspect
@@ -1126,13 +1122,4 @@ class TimeWithZoneMethodsForString < ActiveSupport::TestCase
     assert_raise(ArgumentError) { @u.in_time_zone(Object.new) }
     assert_raise(ArgumentError) { @z.in_time_zone(Object.new) }
   end
-
-  protected
-    def with_tz_default(tz = nil)
-      old_tz = Time.zone
-      Time.zone = tz
-      yield
-    ensure
-      Time.zone = old_tz
-    end
 end
