@@ -1,6 +1,6 @@
 require 'abstract_unit'
 
-class LoadPathsTest < Test::Unit::TestCase
+class LoadPathsTest < ActiveSupport::TestCase
   def test_uniq_load_paths
     load_paths_count = $LOAD_PATH.inject({}) { |paths, path|
       expanded_path = File.expand_path(path)
@@ -10,7 +10,7 @@ class LoadPathsTest < Test::Unit::TestCase
     }
     load_paths_count[File.expand_path('../../lib', __FILE__)] -= 1
 
-    filtered = load_paths_count.select { |k, v| v > 1 }
-    assert filtered.empty?, filtered.inspect
+    load_paths_count.select! { |k, v| v > 1 }
+    assert load_paths_count.empty?, load_paths_count.inspect
   end
 end
