@@ -3,6 +3,7 @@ if RUBY_PLATFORM =~ /java/
   require 'active_support/xml_mini'
   require 'active_support/core_ext/hash/conversions'
 
+
   class JDOMEngineTest < ActiveSupport::TestCase
     include ActiveSupport
 
@@ -175,10 +176,11 @@ if RUBY_PLATFORM =~ /java/
     end
 
     private
-    def assert_equal_rexml(xml)
-      hash = XmlMini.with_backend('REXML') { XmlMini.parse(xml) }
-      assert_equal(hash, XmlMini.parse(xml))
-    end
+      def assert_equal_rexml(xml)
+        parsed_xml = XmlMini.parse(xml)
+        hash = XmlMini.with_backend('REXML') { XmlMini.parse(xml) }
+        assert_equal(hash, parsed_xml)
+      end
   end
 
 else
