@@ -1,5 +1,6 @@
 require 'generators/generators_test_helper'
 require 'rails/generators/rails/model/model_generator'
+require 'active_support/core_ext/string/strip'
 
 class ModelGeneratorTest < Rails::Generators::TestCase
   include GeneratorsTestHelper
@@ -34,6 +35,13 @@ class ModelGeneratorTest < Rails::Generators::TestCase
     assert_no_migration "db/migrate/create_accounts.rb"
   end
 
+  def test_plural_names_are_singularized
+    content = run_generator ["accounts".freeze]
+    assert_file "app/models/account.rb", /class Account < ActiveRecord::Base/
+    assert_file "test/models/account_test.rb", /class AccountTest/
+    assert_match(/\[WARNING\] The model name 'accounts' was recognized as a plural, using the singular 'account' instead\. Override with --force-plural or setup custom inflection rules for this noun before running the generator\./, content)
+  end
+
   def test_model_with_underscored_parent_option
     run_generator ["account", "--parent", "admin/account"]
     assert_file "app/models/account.rb", /class Account < Admin::Account/
@@ -215,7 +223,7 @@ class ModelGeneratorTest < Rails::Generators::TestCase
 
   def test_migration_with_timestamps
     run_generator
-    assert_migration "db/migrate/create_accounts.rb", /t.timestamps/
+    assert_migration "db/migrate/create_accounts.rb", /t.timestamps null: false/
   end
 
   def test_migration_timestamps_are_skipped
@@ -356,6 +364,80 @@ class ModelGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_required_belongs_to_adds_required_association
+    run_generator ["account", "supplier:references{required}"]
+
+    expected_file = <<-FILE.strip_heredoc
+    class Account < ActiveRecord::Base
+      belongs_to :supplier, required: true
+    end
+    FILE
+    assert_file "app/models/account.rb", expected_file
+  end
+
+  def test_required_polymorphic_belongs_to_generages_correct_model
+    run_generator ["account", "supplier:references{required,polymorphic}"]
+
+    expected_file = <<-FILE.strip_heredoc
+    class Account < ActiveRecord::Base
+      belongs_to :supplier, polymorphic: true, required: true
+    end
+    FILE
+    assert_file "app/models/account.rb", expected_file
+  end
+
+  def test_required_and_polymorphic_are_order_independent
+    run_generator ["account", "supplier:references{polymorphic.required}"]
+
+    expected_file = <<-FILE.strip_heredoc
+    class Account < ActiveRecord::Base
+      belongs_to :supplier, polymorphic: true, required: true
+    end
+    FILE
+    assert_file "app/models/account.rb", expected_file
+  end
+
+  def test_required_adds_null_false_to_column
+    run_generator ["account", "supplier:references{required}"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_match(/t\.references :supplier,.*\snull: false/, up)
+      end
+    end
+  end
+
+  def test_foreign_key_is_not_added_for_non_references
+    run_generator ["account", "supplier:string"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_no_match(/foreign_key/, up)
+      end
+    end
+  end
+
+  def test_foreign_key_is_added_for_references
+    run_generator ["account", "supplier:belongs_to", "user:references"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_match(/t\.belongs_to :supplier,.*\sforeign_key: true/, up)
+        assert_match(/t\.references :user,.*\sforeign_key: true/, up)
+      end
+    end
+  end
+
+  def test_foreign_key_is_skipped_for_polymorphic_references
+    run_generator ["account", "supplier:belongs_to{polymorphic}"]
+
+    assert_migration "db/migrate/create_accounts.rb" do |m|
+      assert_method :change, m do |up|
+        assert_no_match(/foreign_key/, up)
+      end
+    end
+  end
+
   private
     def assert_generated_fixture(path, parsed_contents)
       fixture_file = File.new File.expand_path(path, destination_root)
