@@ -1,6 +1,7 @@
 require 'generators/generators_test_helper'
 require 'rails/generators/rails/plugin/plugin_generator'
 require 'generators/shared_generator_tests'
+require 'mocha/setup' # FIXME: stop using mocha
 
 DEFAULT_PLUGIN_FILES = %w(
   .gitignore
@@ -68,14 +69,17 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "test/integration/navigation_test.rb", /ActionDispatch::IntegrationTest/
   end
 
-  def test_inclusion_of_debugger
+  def test_inclusion_of_a_debugger
     run_generator [destination_root, '--full']
     if defined?(JRUBY_VERSION)
       assert_file "Gemfile" do |content|
+        assert_no_match(/byebug/, content)
         assert_no_match(/debugger/, content)
       end
-    else
+    elsif RUBY_VERSION < '2.0.0'
       assert_file "Gemfile", /# gem 'debugger'/
+    else
+      assert_file "Gemfile", /# gem 'byebug'/
     end
   end
 
@@ -95,7 +99,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
   end
 
   def test_generating_adds_dummy_app_without_javascript_and_assets_deps
-    run_generator [destination_root]
+    run_generator
 
     assert_file "test/dummy/app/assets/stylesheets/application.css"
 
@@ -156,13 +160,11 @@ class PluginGeneratorTest < Rails::Generators::TestCase
 
   def test_generation_runs_bundle_install_with_full_and_mountable
     result = run_generator [destination_root, "--mountable", "--full", "--dev"]
+    assert_match(/run  bundle install/, result)
+    assert $?.success?, "Command failed: #{result}"
     assert_file "#{destination_root}/Gemfile.lock" do |contents|
       assert_match(/bukkits/, contents)
     end
-    assert_match(/run  bundle install/, result)
-    assert_match(/Using bukkits \(?0\.0\.1\)?/, result)
-    assert_match(/Your bundle is complete/, result)
-    assert_equal 1, result.scan("Your bundle is complete").size
   end
 
   def test_skipping_javascripts_without_mountable_option
@@ -245,6 +247,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_file "test/test_helper.rb" do |content|
       assert_match(/ActiveRecord::Migrator\.migrations_paths.+\.\.\/test\/dummy\/db\/migrate/, content)
       assert_match(/ActiveRecord::Migrator\.migrations_paths.+<<.+\.\.\/db\/migrate/, content)
+      assert_match(/ActionDispatch::IntegrationTest\.fixture_path = ActiveSupport::TestCase\.fixture_pat/, content)
     end
   end
 
@@ -326,7 +329,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_no_file "bukkits.gemspec"
     assert_file "Gemfile" do |contents|
       assert_no_match('gemspec', contents)
-      assert_match(/gem "rails", "~> #{Rails.version}"/, contents)
+      assert_match(/gem 'rails', '~> #{Rails.version}'/, contents)
       assert_match_sqlite3(contents)
       assert_no_match(/# gem "jquery-rails"/, contents)
     end
@@ -337,7 +340,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     assert_no_file "bukkits.gemspec"
     assert_file "Gemfile" do |contents|
       assert_no_match('gemspec', contents)
-      assert_match(/gem "rails", "~> #{Rails.version}"/, contents)
+      assert_match(/gem 'rails', '~> #{Rails.version}'/, contents)
       assert_match_sqlite3(contents)
     end
   end
@@ -348,7 +351,7 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     Object.const_set('APP_PATH', Rails.root)
     FileUtils.touch gemfile_path
 
-    run_generator [destination_root]
+    run_generator
 
     assert_file gemfile_path, /gem 'bukkits', path: 'tmp\/bukkits'/
   ensure
@@ -385,6 +388,40 @@ class PluginGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_git_name_and_email_in_gemspec_file
+    name = `git config user.name`.chomp rescue "TODO: Write your name"
+    email = `git config user.email`.chomp rescue "TODO: Write your email address"
+
+    run_generator
+    assert_file "bukkits.gemspec" do |contents|
+      assert_match name, contents
+      assert_match email, contents
+    end
+  end
+
+  def test_git_name_in_license_file
+    name = `git config user.name`.chomp rescue "TODO: Write your name"
+
+    run_generator
+    assert_file "MIT-LICENSE" do |contents|
+      assert_match name, contents
+    end
+  end
+
+  def test_no_details_from_git_when_skip_git
+    name = "TODO: Write your name"
+    email = "TODO: Write your email address"
+
+    run_generator [destination_root, '--skip-git']
+    assert_file "MIT-LICENSE" do |contents|
+      assert_match name, contents
+    end
+    assert_file "bukkits.gemspec" do |contents|
+      assert_match name, contents
+      assert_match email, contents
+    end
+  end
+
 protected
   def action(*args, &block)
     silence(:stdout){ generator.send(*args, &block) }
@@ -395,10 +432,10 @@ protected
   end
 
   def assert_match_sqlite3(contents)
-    unless defined?(JRUBY_VERSION)
-      assert_match(/group :development do\n  gem "sqlite3"\nend/, contents)
+    if defined?(JRUBY_VERSION)
+      assert_match(/group :development do\n  gem 'activerecord-jdbcsqlite3-adapter'\nend/, contents)
     else
-      assert_match(/group :development do\n  gem "activerecord-jdbcsqlite3-adapter"\nend/, contents)
+      assert_match(/group :development do\n  gem 'sqlite3'\nend/, contents)
     end
   end
 end
